/*
 * IRalarm
 *
 * Created: 05.05.2013 11:04:19
 * Author: Alex Handfest
 * 
 * 
 * PB1		LED IR			- momentan mit 3,3K
 * 
 * PC1		TSOP_in
 * PC4 AD4	Poti2
 * PC5 AD5	Poti1
 * 
 * PD1		Buzzer
 * PD2		LED rot
 * 
 * www.alex-handfest.de/avr
 */ 

#ifndef F_CPU
#define F_CPU			1000000		// 1 MHz
#endif


#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include "adc.h"


#define LED_RT_ON		(PORTD |=  (1<<PD2))
#define LED_RT_OFF		(PORTD &= ~(1<<PD2))
#define LED_IR_ON		(DDRB  |=  (1<<PB1))
#define LED_IR_OFF		(DDRB  &= ~(1<<PB1))
#define BUZZER_ON		(PORTD |=  (1<<PD1))
#define BUZZER_OFF		(PORTD &= ~(1<<PD1))


// Variablen

unsigned char empfang = 0;		// zhler fr gltigen empfang
unsigned char checken = 0;		// zhler fr gltigen empfangs-block

unsigned char piepsen = 3;		// zeitlnge des alarms
unsigned char sensibl = 3;		// anzahl der checks
unsigned char periode = 3;		// zeitausgleich fr checks


// checkzeit sollte immer eine sekunde sein
// sensible hoch: checken runter, ocr1a hoch


int main(void){
	
	_delay_ms(200);
	
	// 30kHz Trger
	// Timer1 init
	TCCR1A = (1<<COM1A1) | (1<<COM1A0) | (1<<WGM11);
	TCCR1B = (1<<WGM12) | (1<<WGM13) | (1<<CS10);
	// Timer Mode 14: Fast PWM, TOP = ICR1
	// Prescaler: 1
	OCR1A = 15;
	ICR1 = 33;
	// Takt an OC1A (PB1) = Clock / Prescaler / ICR1
	// 23 -> 43,5 kHz
	// 24 -> 41,7 kHz
	// 25 -> 40,0 kHz
	// 26 -> 38,4 kHz
	// 27 -> 37,0 kHz
	// 28 -> 35,7 kHz (TSOP31236)
	// 29 -> 34,5 kHz
	// 30 -> 33,3 kHz
	// 33 -> 30,3 kHz
	
	// TSOP31230 ist intern auf high -> bei Empfang auf low
	
	
	// ADC
	ADC_init();
	_delay_ms(10);
	
	piepsen = (255- (unsigned char) ADC_Read_Avg(5,4));		// Lnge des Alamrs
	if (piepsen < 3) piepsen = 3;
	
	OCR1A = ADC_Read_Avg(4,4) / 12;							// Empfindlichkeit
	if (OCR1A < 2) OCR1A = 2;
	else if (OCR1A > 18) OCR1A = 18;
	sensibl = (255- (unsigned char) ADC_Read_Avg(4,4)) / 20;
	if (sensibl < 4) {
		sensibl = 3;
		periode = 33;
	}	
	else if (sensibl == 4) periode = 25;
	else if (sensibl == 5) periode = 20;
	else if (sensibl == 6) periode = 16;
	else if (sensibl == 7) periode = 14;
	else if (sensibl == 8) periode = 12;
	else if (sensibl == 9) periode = 11;
	else if (sensibl > 9) {
		sensibl = 10;
		periode = 10;
	}	
	
	ADCSRA &= ~(1<<ADEN);						// ADC deaktivieren
	
	
	
	// Ausgnge
	//DDRB |= (1<<PB1); 						//OC1A Ausgang fr Timer1
	DDRD |= (1<<PD1) | (1<<PD2);				// Buzzer und LED rot
	
	// Eingnge
	DDRC  &= ~(1<<PC1) | (1<<PC4) | (1<<PC5);	// TSOP und Potis
	PORTC |=  (1<<PC1);
	
	
	LED_RT_ON;
	
	BUZZER_ON;
	_delay_ms(20);
	BUZZER_OFF;
	_delay_ms(100);
	BUZZER_ON;
	_delay_ms(20);
	BUZZER_OFF;
	
	_delay_ms(100);
	LED_RT_OFF;

	//Globale Interrupts einschalten
	//sei();


	//LED_IR_ON;

    while(1){
		
		// 10x Senden und prfen
		for (int i=0;i<10;i++){
			
			LED_IR_ON;
			_delay_us(400);
			
			if (!(PINC & (1<<PC1))) empfang++;		// low	
			else empfang = 0;
			
			LED_IR_OFF;
			_delay_us(400);
			
			if (  PINC & (1<<PC1) ) empfang++;		// high
			else empfang = 0;
		}
		
		// gltiger Empang
		if (empfang == 20) checken++;
		else {			
			LED_RT_OFF;
			checken = 0;
		}
					
		empfang = 0;
		
		// Zeitverzgerung fr main Wiederholung
		for (int i=0;i<periode;i++) _delay_ms(10);
			
		
		if (checken == sensibl){
			
			LED_RT_ON;
			checken = 0;
			
			for (int i=0;i<piepsen;i++){
				BUZZER_ON;
				_delay_ms(100);
				BUZZER_OFF;
				_delay_ms(100);
			}
			
			_delay_ms(1000);
			LED_RT_OFF;			
		}		
        //TODO:: Please write your application code 
    }
}